@@ -4,22 +4,37 @@ messages without using the nRF24L01's ACK payloads feature.
 """
 import time
 import board
-import digitalio
+from digitalio import DigitalInOut
 
 # if running this on a ATSAMD21 M0 based board
 # from circuitpython_nrf24l01.rf24_lite import RF24
 from circuitpython_nrf24l01.rf24 import RF24
 
-# change these (digital output) pins accordingly
-ce = digitalio.DigitalInOut(board.D4)
-csn = digitalio.DigitalInOut(board.D5)
+# invalid default values for scoping
+SPI_BUS, CSN_PIN, CE_PIN = (None, None, None)
 
-# using board.SPI() automatically selects the MCU's
-# available SPI pins, board.SCK, board.MOSI, board.MISO
-spi = board.SPI()  # init spi bus object
+try:  # on Linux
+    import spidev
+
+    SPI_BUS = spidev.SpiDev()  # for a faster interface on linux
+    CSN_PIN = 0  # use CE0 on default bus (even faster than using any pin)
+    CE_PIN = DigitalInOut(board.D22)  # using pin gpio22 (BCM numbering)
+
+except ImportError:  # on CircuitPython only
+    # using board.SPI() automatically selects the MCU's
+    # available SPI pins, board.SCK, board.MOSI, board.MISO
+    SPI_BUS = board.SPI()  # init spi bus object
+
+    # change these (digital output) pins accordingly
+    CE_PIN = DigitalInOut(board.D4)
+    CSN_PIN = DigitalInOut(board.D5)
 
 # initialize the nRF24L01 on the spi bus object
-nrf = RF24(spi, csn, ce)
+nrf = RF24(SPI_BUS, CSN_PIN, CE_PIN)
+# On Linux, csn value is a bit coded
+#                 0 = bus 0, CE0  # SPI bus 0 is enabled by default
+#                10 = bus 1, CE0  # enable SPI bus 2 prior to running this
+#                21 = bus 2, CE1  # enable SPI bus 1 prior to running this
 
 # set the Power Amplifier level to -12 dBm since this test example is
 # usually run with nRF24L01 transceivers in close proximity
@@ -72,12 +87,9 @@ def master(count=5):  # count = 5 will only transmit 5 packets
             nrf.listen = False  # put the radio back in TX mode
             end_timer = time.monotonic_ns()  # stop timer
             print(
-                "Transmission successful! Time to transmit: "
-                "{} us. Sent: {}{}".format(
-                    int((end_timer - start_timer) / 1000),
-                    buffer[:6].decode("utf-8"),
-                    counter[0],
-                ),
+                "Transmission successful! Time to transmit:",
+                f"{int((end_timer - start_timer) / 1000)} us. Sent:",
+                "{}{}".format(buffer[:6].decode("utf-8"), counter[0]),
                 end=" ",
             )
             if nrf.pipe is None:  # is there a payload?
@@ -90,12 +102,8 @@ def master(count=5):  # count = 5 will only transmit 5 packets
                 # save new counter from response
                 counter[0] = received[7:8][0]
                 print(
-                    "Receieved {} bytes with pipe {}: {}{}".format(
-                        length,
-                        pipe_number,
-                        bytes(received[:6]).decode("utf-8"),  # convert to str
-                        counter[0],
-                    )
+                    f"Receieved {length} bytes with pipe {pipe_number}:",
+                    "{}{}".format(bytes(received[:6]).decode("utf-8"), counter[0]),
                 )
         count -= 1
         # make example readable in REPL by slowing down transmissions
@@ -122,13 +130,9 @@ def slave(timeout=6):
                 result = nrf.send(b"World \0" + bytes([counter[0]]))
             nrf.listen = True  # put the radio back in RX mode
             print(
-                "Received {} on pipe {}: {}{} Sent:".format(
-                    length,
-                    pipe,
-                    bytes(received[:6]).decode("utf-8"),  # convert to str
-                    received[7:8][0],
-                ),
-                end=" ",
+                f"Received {length} on pipe {pipe}:",
+                "{}{}".format(bytes(received[:6]).decode("utf-8"), received[7:8][0]),
+                end=" Sent: ",
             )
             if not result:
                 print("Response failed or timed out")
@@ -143,10 +147,6 @@ def slave(timeout=6):
 def set_role():
     """Set the role using stdin stream. Timeout arg for slave() can be
     specified using a space delimiter (e.g. 'R 10' calls `slave(10)`)
-
-    :return:
-        - True when role is complete & app should continue running.
-        - False when app should exit
     """
     user_input = (
         input(
@@ -158,16 +158,10 @@ def set_role():
     )
     user_input = user_input.split()
     if user_input[0].upper().startswith("R"):
-        if len(user_input) > 1:
-            slave(int(user_input[1]))
-        else:
-            slave()
+        slave(*[int(x) for x in user_input[1:2]])
         return True
     if user_input[0].upper().startswith("T"):
-        if len(user_input) > 1:
-            master(int(user_input[1]))
-        else:
-            master()
+        master(*[int(x) for x in user_input[1:2]])
         return True
     if user_input[0].upper().startswith("Q"):
         nrf.power = False
