@@ -5,23 +5,38 @@ transceivers. This technique is called "multiceiver" in the datasheet.
 import time
 import struct
 import board
-import digitalio
+from digitalio import DigitalInOut
 
 # if running this on a ATSAMD21 M0 based board
 # from circuitpython_nrf24l01.rf24_lite import RF24
 from circuitpython_nrf24l01.rf24 import RF24
 
-# change these (digital output) pins accordingly
-ce = digitalio.DigitalInOut(board.D4)
-csn = digitalio.DigitalInOut(board.D5)
+# invalid default values for scoping
+SPI_BUS, CSN_PIN, CE_PIN = (None, None, None)
+
+try:  # on Linux
+    import spidev
+
+    SPI_BUS = spidev.SpiDev()  # for a faster interface on linux
+    CSN_PIN = 0  # use CE0 on default bus (even faster than using any pin)
+    CE_PIN = DigitalInOut(board.D22)  # using pin gpio22 (BCM numbering)
+
+except ImportError:  # on CircuitPython only
+    # using board.SPI() automatically selects the MCU's
+    # available SPI pins, board.SCK, board.MOSI, board.MISO
+    SPI_BUS = board.SPI()  # init spi bus object
+
+    # change these (digital output) pins accordingly
+    CE_PIN = DigitalInOut(board.D4)
+    CSN_PIN = DigitalInOut(board.D5)
 
-# using board.SPI() automatically selects the MCU's
-# available SPI pins, board.SCK, board.MOSI, board.MISO
-spi = board.SPI()  # init spi bus object
 
-# we'll be using the dynamic payload size feature (enabled by default)
 # initialize the nRF24L01 on the spi bus object
-nrf = RF24(spi, csn, ce)
+nrf = RF24(SPI_BUS, CSN_PIN, CE_PIN)
+# On Linux, csn value is a bit coded
+#                 0 = bus 0, CE0  # SPI bus 0 is enabled by default
+#                10 = bus 1, CE0  # enable SPI bus 2 prior to running this
+#                21 = bus 2, CE1  # enable SPI bus 1 prior to running this
 
 # set the Power Amplifier level to -12 dBm since this test example is
 # usually run with nRF24L01 transceivers in close proximity
@@ -56,7 +71,7 @@ def base(timeout=10):
             # NOTE read() clears the pipe number and payload length data
             print("Received", nrf.any(), "on pipe", nrf.pipe, end=" ")
             node_id, payload_id = struct.unpack("<ii", nrf.read())
-            print("from node {}. PayloadID: {}".format(node_id, payload_id))
+            print(f"from node {node_id}. PayloadID: {payload_id}")
             start_timer = time.monotonic()  # reset timer with every payload
     nrf.listen = False
 
@@ -85,10 +100,8 @@ def node(node_number=0, count=6):
         # show something to see it isn't frozen
         if report:
             print(
-                "Transmission of payloadID {} as node {} successfull! "
-                "Transmission time: {} us".format(
-                    counter, node_number, (end_timer - start_timer) / 1000
-                )
+                f"Transmission of payloadID {counter} as node {node_number} success"
+                f"full! Transmission time: {int((end_timer - start_timer) / 1000)} us"
             )
         else:
             print("Transmission failed or timed out")
@@ -98,10 +111,6 @@ def node(node_number=0, count=6):
 def set_role():
     """Set the role using stdin stream. Node number arg for node() can be
     specified using a space delimiter (e.g. 'T 0' calls `node(0)`)
-
-    :return:
-        - True when role is complete & app should continue running.
-        - False when app should exit
     """
     user_input = (
         input(
@@ -113,18 +122,10 @@ def set_role():
     )
     user_input = user_input.split()
     if user_input[0].upper().startswith("R"):
-        if len(user_input) > 1:
-            base(int(user_input[1]))
-        else:
-            base()
+        base(*[int(x) for x in user_input[1:2]])
         return True
     if user_input[0].upper().startswith("T"):
-        if len(user_input) > 2:
-            node(int(user_input[1]), int(user_input[2]))
-        elif len(user_input) > 1:
-            node(int(user_input[1]))
-        else:
-            node()
+        node(*[int(x) for x in user_input[1:3]])
         return True
     if user_input[0].upper().startswith("Q"):
         nrf.power = False
@@ -146,5 +147,6 @@ else:
     print(
         "    Run base() on the receiver\n    "
         "Run node(node_number) on a transmitter\n    "
-        "node()'s parameter, `node_number`, must be in range [0, 5]"""
+        "node()'s parameter, `node_number`, must be in range [0, 5]"
+        ""
     )
