@@ -3,23 +3,38 @@ Example of library usage for streaming multiple payloads.
 """
 import time
 import board
-import digitalio
+from digitalio import DigitalInOut
 
 # if running this on a ATSAMD21 M0 based board
 # from circuitpython_nrf24l01.rf24_lite import RF24
 from circuitpython_nrf24l01.rf24 import RF24
 
-# change these (digital output) pins accordingly
-ce = digitalio.DigitalInOut(board.D4)
-csn = digitalio.DigitalInOut(board.D5)
+# invalid default values for scoping
+SPI_BUS, CSN_PIN, CE_PIN = (None, None, None)
+
+try:  # on Linux
+    import spidev
+
+    SPI_BUS = spidev.SpiDev()  # for a faster interface on linux
+    CSN_PIN = 0  # use CE0 on default bus (even faster than using any pin)
+    CE_PIN = DigitalInOut(board.D22)  # using pin gpio22 (BCM numbering)
+
+except ImportError:  # on CircuitPython only
+    # using board.SPI() automatically selects the MCU's
+    # available SPI pins, board.SCK, board.MOSI, board.MISO
+    SPI_BUS = board.SPI()  # init spi bus object
+
+    # change these (digital output) pins accordingly
+    CE_PIN = DigitalInOut(board.D4)
+    CSN_PIN = DigitalInOut(board.D5)
 
-# using board.SPI() automatically selects the MCU's
-# available SPI pins, board.SCK, board.MOSI, board.MISO
-spi = board.SPI()  # init spi bus object
 
-# we'll be using the dynamic payload size feature (enabled by default)
 # initialize the nRF24L01 on the spi bus object
-nrf = RF24(spi, csn, ce)
+nrf = RF24(SPI_BUS, CSN_PIN, CE_PIN)
+# On Linux, csn value is a bit coded
+#                 0 = bus 0, CE0  # SPI bus 0 is enabled by default
+#                10 = bus 1, CE0  # enable SPI bus 2 prior to running this
+#                21 = bus 2, CE1  # enable SPI bus 1 prior to running this
 
 # set the Power Amplifier level to -12 dBm since this test example is
 # usually run with nRF24L01 transceivers in close proximity
@@ -43,7 +58,7 @@ nrf.open_rx_pipe(1, address[not radio_number])  # using pipe 1
 
 # uncomment the following 2 lines for compatibility with TMRh20 library
 # nrf.allow_ask_no_ack = False
-# nrf.dynamic_payloads = False
+nrf.dynamic_payloads = False
 
 
 def make_buffers(size=32):
@@ -79,37 +94,35 @@ def master(count=1, size=32):  # count = 5 will transmit the list 5 times
         for r in result:  # tally the resulting success rate
             successful += 1 if r else 0
     print(
-        "successfully sent {}% ({}/{})".format(
-            successful / (size * count) * 100, successful, size * count
-        )
+        f"successfully sent {successful / (size * count) * 100}%",
+        f"({successful}/{size * count})"
     )
 
 
 def master_fifo(count=1, size=32):
     """Similar to the `master()` above except this function uses the full
     TX FIFO and `RF24.write()` instead of `RF24.send()`"""
-    if size < 6:
-        print("setting size to 6;", size, "is not allowed for this test.")
-        size = 6
     buf = make_buffers(size)  # make a list of payloads
     nrf.listen = False  # ensures the nRF24L01 is in TX mode
-    for c in range(count):  # transmit the same payloads this many times
+    for cnt in range(count):  # transmit the same payloads this many times
         nrf.flush_tx()  # clear the TX FIFO so we can use all 3 levels
         # NOTE the write_only parameter does not initiate sending
         buf_iter = 0  # iterator of payloads for the while loop
         failures = 0  # keep track of manual retries
         start_timer = time.monotonic_ns()  # start timer
         while buf_iter < size:  # cycle through all the payloads
+            nrf.ce_pin = False
             while buf_iter < size and nrf.write(buf[buf_iter], write_only=1):
                 # NOTE write() returns False if TX FIFO is already full
                 buf_iter += 1  # increment iterator of payloads
-            ce.value = True  # start tranmission (after 10 microseconds)
+            nrf.ce_pin = True
             while not nrf.fifo(True, True):  # updates irq_df flag
                 if nrf.irq_df:
                     # reception failed; we need to reset the irq_rf flag
-                    ce.value = False  # fall back to Standby-I mode
+                    nrf.ce_pin = False  # fall back to Standby-I mode
                     failures += 1  # increment manual retries
-                    if failures > 99 and buf_iter < 7 and c < 2:
+                    nrf.clear_status_flags()  # clear the irq_df flag
+                    if failures > 99 and buf_iter < 7 and cnt < 2:
                         # we need to prevent an infinite loop
                         print(
                             "Make sure slave() node is listening."
@@ -117,26 +130,14 @@ def master_fifo(count=1, size=32):
                         )
                         buf_iter = size + 1  # be sure to exit the while loop
                         nrf.flush_tx()  # discard all payloads in TX FIFO
-                        break
-                    nrf.clear_status_flags()  # clear the irq_df flag
-                    ce.value = True  # start re-transmitting
-            ce.value = False
+                    else:
+                        nrf.ce_pin = True  # start re-transmitting
+        nrf.ce_pin = False
         end_timer = time.monotonic_ns()  # end timer
         print(
-            "Transmission took {} us with {} failures detected.".format(
-                (end_timer - start_timer) / 1000, failures
-            ),
-            end=" " if failures < 100 else "\n",
+            f"Transmission took {(end_timer - start_timer) / 1000} us",
+            f"with {failures} failures detected."
         )
-        if 1 <= failures < 100:
-            print(
-                "\n\nHINT: Try playing with the 'ard' and 'arc' attributes to"
-                " reduce the number of\nfailures detected. Tests were better"
-                " with these attributes at higher values, but\nnotice the "
-                "transmission time differences."
-            )
-        elif not failures:
-            print("You Win!")
 
 
 def slave(timeout=5):
@@ -149,7 +150,7 @@ def slave(timeout=5):
             count += 1
             # retreive the received packet's payload
             buffer = nrf.read()  # clears flags & empties RX FIFO
-            print("Received: {} - {}".format(buffer, count))
+            print(f"Received: {buffer} - {count}")
             start_timer = time.monotonic()  # reset timer on every RX payload
 
     # recommended behavior is to keep in TX mode while idle
@@ -159,44 +160,25 @@ def slave(timeout=5):
 def set_role():
     """Set the role using stdin stream. Timeout arg for slave() can be
     specified using a space delimiter (e.g. 'R 10' calls `slave(10)`)
-
-    :return:
-        - True when role is complete & app should continue running.
-        - False when app should exit
     """
     user_input = (
         input(
             "*** Enter 'R' for receiver role.\n"
-            "*** Enter 'T' for transmitter role (using 1 level"
-            " of the TX FIFO).\n"
-            "*** Enter 'F' for transmitter role (using all 3 levels"
-            " of the TX FIFO).\n"
+            "*** Enter 'T' for transmitter role (using 1 level  of the TX FIFO).\n"
+            "*** Enter 'F' for transmitter role (using all 3 levels of the TX FIFO).\n"
             "*** Enter 'Q' to quit example.\n"
         )
         or "?"
     )
     user_input = user_input.split()
     if user_input[0].upper().startswith("R"):
-        if len(user_input) > 1:
-            slave(int(user_input[1]))
-        else:
-            slave()
+        slave(*[int(x) for x in user_input[1:2]])
         return True
     if user_input[0].upper().startswith("T"):
-        if len(user_input) > 2:
-            master(int(user_input[1]), int(user_input[2]))
-        elif len(user_input) > 1:
-            master(int(user_input[1]))
-        else:
-            master()
+        master(*[int(x) for x in user_input[1:3]])
         return True
     if user_input[0].upper().startswith("F"):
-        if len(user_input) > 2:
-            master_fifo(int(user_input[1]), int(user_input[2]))
-        elif len(user_input) > 1:
-            master_fifo(int(user_input[1]))
-        else:
-            master_fifo()
+        master_fifo(*[int(x) for x in user_input[1:3]])
         return True
     if user_input[0].upper().startswith("Q"):
         nrf.power = False
@@ -215,8 +197,6 @@ if __name__ == "__main__":
         print(" Keyboard Interrupt detected. Powering down radio...")
         nrf.power = False
 else:
-    print(
-        "    Run slave() on receiver\n    Run master() on transmitter to use"
-        " 1 level of the TX FIFO\n    Run master_fifo() on transmitter to use"
-        " all 3 levels of the TX FIFO"
-    )
+    print("    Run slave() on receiver")
+    print("    Run master() on transmitter to use 1 level of the TX FIFO")
+    print("    Run master_fifo() on transmitter to use all 3 levels of the TX FIFO")
